@@ -12,11 +12,9 @@ from pyspedas import maven
 #         self.assertTrue(os.path.join(os.path.join(os.path.dirname(__file__), '..'), 'maven_orb_rec.orb'))
 
 class LoadTestCases(unittest.TestCase):
-    # Kp data stopped loading ~November 2020
-    # need to investigate why - egrimes
-    # def test_load_kp_data(self):
-    #     data = maven.kp()
-    #     self.assertTrue(data_exists('mvn_kp::spacecraft::geo_x'))
+    def test_load_kp_data(self):
+        data = maven.kp()
+        self.assertTrue(data_exists('mvn_kp::spacecraft::geo_x'))
 
     def test_load_mag_data(self):
         data = maven.mag()
@@ -39,9 +37,9 @@ class LoadTestCases(unittest.TestCase):
         data = maven.sep()
         self.assertTrue(data_exists('f_ion_flux_tot_s2-cal-svy-full'))
 
-    def test_load_lpw_data(self):
-        data = maven.lpw()
-        self.assertTrue(data_exists('mvn_lpw_lp_iv_l2_lpiv'))
+    #def test_load_lpw_data(self):
+    #    data = maven.lpw()
+    #    self.assertTrue(data_exists('mvn_lpw_lp_iv_l2_lpiv'))
 
     def test_load_euv_data(self):
         data = maven.euv()
