@@ -1,8 +1,10 @@
 
 from .load import load
+from .filter import filter_fields
 
-from pytplot import options
+from pytplot import options, data_quants
 
+# Loading
 def fields(trange=['2018-11-5', '2018-11-6'], 
         datatype='mag_rtn', 
         level='l2',
@@ -13,7 +15,10 @@ def fields(trange=['2018-11-5', '2018-11-6'],
         downloadonly=False,
         notplot=False,
         no_update=False,
-        time_clip=False):
+        time_clip=False,
+        username=None,
+        password=None
+        ):
     """
     This function loads Parker Solar Probe FIELDS data
     
@@ -25,7 +30,26 @@ def fields(trange=['2018-11-5', '2018-11-6'],
             ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']
 
         datatype: str
-            Data type; Valid options:
+            Data type; Valid options include:
+                'mag_RTN'
+                'mag_RTN_1min'
+                'mag_rtn_4_per_cycle' (SPDF only)
+                'mag_RTN_4_Sa_per_Cyc' 
+                'mag_SC'
+                'mag_SC_1min'
+                'mag_sc_4_per_cycle' (SPDF only)
+                'mag_SC_4_Sa_per_Cyc' 
+                'mag_VSO' (limited dates)
+                'rfs_burst' (limited dates)
+                'rfs_hfr', 
+                'rfs_lfr'
+                'f2_100bps'
+                'dfb_dc_spec'
+                'dfb_ac_spec'
+                'dfb_dc_xspec'
+                'dfb_ac_xspec'
+                'merged_scam_wf'
+                'sqtn_rfs_V1V2'
 
         suffix: str
             The tplot variable names will be given this suffix.  By default, 
@@ -33,8 +57,9 @@ def fields(trange=['2018-11-5', '2018-11-6'],
 
         get_support_data: bool
             Data with an attribute "VAR_TYPE" with a value of "support_data"
-            will be loaded into tplot.  By default, only loads in data with a 
-            "VAR_TYPE" attribute of "data".
+            will be loaded into tplot.  By default, this flag is False but 
+            FIELDS support data is always loaded for datatypes where filtering
+            on quality flags is supported.
 
         varformat: str
             The file variable formats to load into tplot.  Wildcard character
@@ -57,18 +82,40 @@ def fields(trange=['2018-11-5', '2018-11-6'],
         time_clip: bool
             Time clip the variables to exactly the range specified in the trange keyword
 
+        username: str
+            Username to use for authentication.
+
+            If passed, attempt to download data from the FIELDS Instrument Team server
+            instead of the fully public server at SPDF.
+            Provides access to unpublished, V02 files.
+
+            Implemented for dataypes:
+                'mag_RTN_1min'
+                'mag_RTN_4_Sa_per_Cyc'
+                'mag_SC'
+                'mag_SC_1min'
+                'mag_SC_4_Sa_per_Cyc'
+                'sqtn_rfs_V1V2'
+
+        password: str
+            Password to use for authentication
+
     Returns
     ----------
         List of tplot variables created.
 
     """
 
-    # SCaM data is Level 3
-    if datatype == 'merged_scam_wf':
+    if suffix:
+        suffix = '_' + suffix
+
+    # SCaM and QTN data are Level 3
+    if datatype.lower() in ['merged_scam_wf', 'sqtn_rfs_v1v2']:
         level = 'l3'
+        print("Using LEVEL=L3")
 
     spec_types = None
-    if datatype == 'dfb_dc_spec' or datatype == 'dfb_ac_spec' or datatype == 'dfb_dc_xspec' or datatype == 'dfb_ac_xspec':
+    if datatype in ['dfb_dc_spec', 'dfb_ac_spec', 'dfb_dc_xspec', 'dfb_ac_xspec']:
         if level == 'l1':
             spec_types = ['1', '2', '3', '4']
         else:
@@ -84,16 +131,46 @@ def fields(trange=['2018-11-5', '2018-11-6'],
                     'SCMulfhg_SCMvlfhg','SCMulfhg_SCMwlfhg','SCMvlfhg_SCMwlfhg',
                     'dV12hg_dV34hg']
 
-    loaded_vars = load(instrument='fields', trange=trange, datatype=datatype, spec_types=spec_types, level=level, suffix=suffix, get_support_data=get_support_data, varformat=varformat, varnames=varnames, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)
+    loaded_vars = load(
+        instrument='fields', trange=trange, datatype=datatype, spec_types=spec_types, level=level, 
+        suffix=suffix, get_support_data=get_support_data, varformat=varformat, varnames=varnames, 
+        downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update,
+        username=username, password=password
+    )
     
     if loaded_vars is None or notplot or downloadonly:
         return loaded_vars
 
-    if 'psp_fld_l2_mag_RTN'+suffix in loaded_vars:
-        options('psp_fld_l2_mag_RTN'+suffix, 'legend_names', ['Br (RTN)', 'Bt (RTN)', 'Bn (RTN)'])
+    # If variables are loaded that quality flag filtering supports --
+    # Make sure the quality flag variable is also loaded and linked. 
+    mag_rtnvars = [x for x in loaded_vars if 'fld_l2_mag_RTN' in x ]
+    mag_scvars = [x for x in loaded_vars if 'fld_l2_mag_SC' in x ]
+    rfs_vars = [x for x in loaded_vars if 'rfs_lfr' in x or 'rfs_hfr' in x]
+    if (len(mag_rtnvars + mag_scvars + rfs_vars) > 0) \
+        & ('psp_fld_l2_quality_flags'+suffix not in loaded_vars):
+        loaded_extra = load(
+            instrument='fields', trange=trange, datatype=datatype, spec_types=spec_types, level=level, 
+            suffix=suffix, get_support_data=True, varformat=varformat, varnames=['psp_fld_l2_quality_flags'], 
+            downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update,
+            username=username, password=password
+        )
+        qf_root = 'psp_fld_l2_quality_flags'+suffix if 'psp_fld_l2_quality_flags'+suffix in loaded_extra else None
+        loaded_vars += loaded_extra
+
+    for var in mag_rtnvars:
+        options(var, 'legend_names', ['Br (RTN)', 'Bt (RTN)', 'Bn (RTN)'])
+        data_quants[var] = data_quants[var].assign_attrs({'qf_root':qf_root})
+
+    for var in mag_scvars:
+        options(var, 'legend_names', ['Bx', 'By', 'Bz'])
+        data_quants[var] = data_quants[var].assign_attrs({'qf_root':qf_root})
+
+    for var in rfs_vars:
+        data_quants[var] = data_quants[var].assign_attrs({'qf_root':qf_root})
 
     return loaded_vars
 
+
 def spc(trange=['2018-11-5', '2018-11-6'], 
         datatype='l3i', 
         level='l3',
@@ -104,7 +181,10 @@ def spc(trange=['2018-11-5', '2018-11-6'],
         downloadonly=False,
         notplot=False,
         no_update=False,
-        time_clip=False):
+        time_clip=False,
+        username=None,
+        password=None
+    ):
     """
     This function loads Parker Solar Probe Solar Probe Cup data
     
@@ -116,7 +196,9 @@ def spc(trange=['2018-11-5', '2018-11-6'],
             ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']
 
         datatype: str
-            Data type; Valid options:
+            Data type; Valid options include:
+                'l3i' (level='l3')
+                'l2i' (level='l2')
 
         suffix: str
             The tplot variable names will be given this suffix.  By default, 
@@ -148,12 +230,32 @@ def spc(trange=['2018-11-5', '2018-11-6'],
         time_clip: bool
             Time clip the variables to exactly the range specified in the trange keyword
 
+        username: str
+            Username to use for authentication.
+            
+            If passed, attempt to download data from the SWEAP Instrument Team server
+            instead of the fully public server at SPDF.
+            Provides access to unpublished files.
+
+        password: str
+            Password to use for authentication
+
     Returns
     ----------
         List of tplot variables created.
 
     """
-    return load(instrument='spc', trange=trange, datatype=datatype, level=level, suffix=suffix, get_support_data=get_support_data, varformat=varformat, varnames=varnames, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)
+
+    if datatype == 'l3i':
+        level = 'l3'
+        print("Using LEVEL=L3")
+    elif datatype == 'l2i':
+        level = 'l2'
+        print("Using LEVEL=L2")
+
+    return load(instrument='spc', trange=trange, datatype=datatype, level=level, suffix=suffix, 
+        get_support_data=get_support_data, varformat=varformat, varnames=varnames, downloadonly=downloadonly, 
+        notplot=notplot, time_clip=time_clip, no_update=no_update, username=username, password=password)
 
 def spe(trange=['2018-11-5', '2018-11-6'], 
         datatype='spa_sf1_32e', 
@@ -165,7 +267,8 @@ def spe(trange=['2018-11-5', '2018-11-6'],
         downloadonly=False,
         notplot=False,
         no_update=False,
-        time_clip=False):
+        time_clip=False
+    ):
     """
     This function loads Parker Solar Probe SWEAP/SPAN-e data
     
@@ -177,7 +280,14 @@ def spe(trange=['2018-11-5', '2018-11-6'],
             ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']
 
         datatype: str
-            Data type; Valid options:
+            Data type; Valid options include:
+                spa_sf0_pad  (L3)
+                spb_sf0_pad  (L3)
+                spe_sf0_pad  (L3)
+                spa_sf1_32e  (L2)
+                spb_sf1_32e  (L2)
+                spa_sf0_16ax8dx32e  (L2)
+                spb_sf0_16ax8dx32e  (L2)
 
         suffix: str
             The tplot variable names will be given this suffix.  By default, 
@@ -214,10 +324,12 @@ def spe(trange=['2018-11-5', '2018-11-6'],
         List of tplot variables created.
 
     """
-    return load(instrument='spe', trange=trange, datatype=datatype, level=level, suffix=suffix, get_support_data=get_support_data, varformat=varformat, varnames=varnames, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)
+    return load(instrument='spe', trange=trange, datatype=datatype, level=level, 
+        suffix=suffix, get_support_data=get_support_data, varformat=varformat, varnames=varnames, 
+        downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)
 
 def spi(trange=['2018-11-5', '2018-11-6'], 
-        datatype='spi_sf0a_mom_inst', 
+        datatype='sf00_l3_mom', 
         level='l3',
         suffix='',  
         get_support_data=False, 
@@ -226,7 +338,10 @@ def spi(trange=['2018-11-5', '2018-11-6'],
         downloadonly=False,
         notplot=False,
         no_update=False,
-        time_clip=False):
+        time_clip=False,
+        username=None,
+        password=None
+    ):
     """
     This function loads Parker Solar Probe SWEAP/SPAN-i data
     
@@ -238,7 +353,11 @@ def spi(trange=['2018-11-5', '2018-11-6'],
             ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']
 
         datatype: str
-            Data type; Valid options:
+            Data type; Valid options Include:
+                'sf00_l3_mom': Moments of the Proton distribution function (RTN)
+                'sf0a_l3_mom': Moments of the Alpha distribution function (RTN)
+                'sf00_l3_mom_inst': Moments of the Proton distribution function (Instrument Frame)
+                'sf0a_l3_mom_inst': Moments of the Alpha distribution function (Instrument Frame)
 
         suffix: str
             The tplot variable names will be given this suffix.  By default, 
@@ -270,12 +389,31 @@ def spi(trange=['2018-11-5', '2018-11-6'],
         time_clip: bool
             Time clip the variables to exactly the range specified in the trange keyword
 
+        username: str
+            Username to use for authentication.
+            
+            If passed, attempt to download data from the SWEAP Instrument Team server
+            instead of the fully public server at SPDF.
+            Provides access to unpublished files.
+
+        password: str
+            Password to use for authentication
+
     Returns
     ----------
         List of tplot variables created.
 
     """
-    return load(instrument='spi', trange=trange, datatype=datatype, level=level, suffix=suffix, get_support_data=get_support_data, varformat=varformat, varnames=varnames, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)
+    if datatype in ['sf00_l3_mom','sf0a_l3_mom','sf00_l3_mom_inst','sf0a_l3_mom_inst']:
+        datatype = 'spi_' + datatype
+        level = 'l3'
+        print("Using LEVEL=L3")
+
+    return load(instrument='spi', trange=trange, datatype=datatype, level=level, 
+        suffix=suffix, get_support_data=get_support_data, varformat=varformat, varnames=varnames, 
+        downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update, 
+        username=username, password=password
+        )
 
 def epihi(trange=['2018-11-5', '2018-11-6'], 
         datatype='let1_rates1h', 
@@ -336,7 +474,9 @@ def epihi(trange=['2018-11-5', '2018-11-6'],
         List of tplot variables created.
 
     """
-    return load(instrument='epihi', trange=trange, datatype=datatype, level=level, suffix=suffix, get_support_data=get_support_data, varformat=varformat, varnames=varnames, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)
+    return load(instrument='epihi', trange=trange, datatype=datatype, level=level, 
+        suffix=suffix, get_support_data=get_support_data, varformat=varformat, varnames=varnames, 
+        downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)
 
 def epilo(trange=['2018-11-5', '2018-11-6'], 
         datatype='pe', 
@@ -397,7 +537,9 @@ def epilo(trange=['2018-11-5', '2018-11-6'],
         List of tplot variables created.
 
     """
-    return load(instrument='epilo', trange=trange, datatype=datatype, level=level, suffix=suffix, get_support_data=get_support_data, varformat=varformat, varnames=varnames, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)
+    return load(instrument='epilo', trange=trange, datatype=datatype, level=level, 
+        suffix=suffix, get_support_data=get_support_data, varformat=varformat, varnames=varnames, 
+        downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)
 
 def epi(trange=['2018-11-5', '2018-11-6'], 
         datatype='summary', 
@@ -458,4 +600,7 @@ def epi(trange=['2018-11-5', '2018-11-6'],
         List of tplot variables created.
 
     """
-    return load(instrument='epi', trange=trange, datatype=datatype, level=level, suffix=suffix, get_support_data=get_support_data, varformat=varformat, varnames=varnames, downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)
+    return load(instrument='epi', trange=trange, datatype=datatype, level=level, 
+        suffix=suffix, get_support_data=get_support_data, varformat=varformat, varnames=varnames, 
+        downloadonly=downloadonly, notplot=notplot, time_clip=time_clip, no_update=no_update)
+
